package main

import (
	"cicv-data-closedloop/common/config/c_log"
	commonConfig "cicv-data-closedloop/pji/common/cfg"
	commonService "cicv-data-closedloop/pji/common/svc"
	controlConfig "cicv-data-closedloop/pji/control/package/cfg"
	"net/rpc"
	"runtime"
	"time"
)

func init() {
	runtime.GOMAXPROCS(1)
	// 初始化日志配置
	c_log.InitLog("pji-control")
	// 初始化本地配置文件（第1处配置，在本地文件）
	commonConfig.InitLocalConfig()
	// 初始化Oss连接信息
	commonConfig.InitOssConfig()
	// 初始化业务逻辑配置信息，配置文件在oss上（第2处配置，在oss文件）
	commonConfig.InitCloudConfig()
	// 首先初始化平台配置。
	commonConfig.InitPlatformConfig()
	// 初始化rpc客户端，用于杀死旧的采集程序
	controlConfig.InitNacos()
}

func main() {

	//  轮询任务接口判断是否有更新
	for {
		time.Sleep(time.Duration(2) * time.Second)
		// 1 获取当前设备的任务的 status
		status, err := commonConfig.GetStatus(commonConfig.PlatformConfig.TaskConfigId)
		if err != nil {
			c_log.GlobalLogger.Error("获取配置status失败：", err)
			continue
		}
		// 2 判断 status
		// UN_CHANGE 没有新的任务，无需更改
		// CHANGE 有新的任务，需要杀死旧的任务并重启
		// NONE 设备没有配置任务，需要杀死旧的任务
		if status == "UN_CHANGE" {
			continue
		} else if status == "CHANGE" || status == "NONE" {
			// 3 发送rpc信号杀死采集程序
			var masterArgs *commonService.KillSignal
			if status == "CHANGE" {
				masterArgs = &commonService.KillSignal{NodeName: "master", DropUploadData: commonConfig.PlatformConfig.DropUploadData, Restart: true}
				c_log.GlobalLogger.Info("更新任务，发送rpc重启信号：", masterArgs)
			}
			if status == "NONE" {
				masterArgs = &commonService.KillSignal{NodeName: "master", DropUploadData: commonConfig.PlatformConfig.DropUploadData, Restart: false}
				c_log.GlobalLogger.Info("杀死任务，发送rpc结束信号：", masterArgs)
			}

			KillRpcClient, err := rpc.Dial("tcp", commonConfig.CloudConfig.Hosts[0].Ip+":"+commonConfig.CloudConfig.RpcPort)
			if err != nil {
				c_log.GlobalLogger.Error("创建rpc客户端连接master失败：", err)
				// 此处关闭client会报错
				continue
			}

			reply := 0
			err = KillRpcClient.Call("KillService.Kill", masterArgs, &reply)
			if err != nil {
				c_log.GlobalLogger.Error("发送 rpc 请求到 master 报错：", err)
				//TODO 这里会报错unexpected EOF但是不影响，先注释 close 和 continue
				//KillRpcClient.Close()
				//continue
			}
			c_log.GlobalLogger.Info("获取数据闭环平台最新配置。")
			commonConfig.InitPlatformConfig()
			KillRpcClient.Close()
		} else {
			c_log.GlobalLogger.Error("未知的采集任务状态。【status】=", status)
		}
	}

}
